// ******************************************************************************
// * Licensed Materials - Property of IBM
// * 5724-W86, 5724-X63
// *  Copyright IBM Corporation 2008, 2011. All Rights Reserved.
// * US Government Users Restricted Rights- Use, duplication or disclosure
// * restricted by GSA ADP Schedule Contract with IBM Corp.
// ******************************************************************************
// *
// ******************************************************************************

/**
 * This creates the aggregator to do delta for all parameters and record the value
 * of the delta on to the record.aggSettings.formulaPath, using the record.CounterID as the
 * metric key and the record.subelement.name as the key to the subelement
 * @param {PV.Operator} operator that is the parent to this aggregator
 * @return {PV.Aggregator} aggregator
 */
function createAggregator(operator){
	var aggregator = operator.Aggregator();
	aggregator.setPeriodDuration(900);
	aggregator.parentOperator=operator;
	aggregator.persistent=true;
	aggregator.name="deltaAggregator";
     
     //Debugging
     // if (LOG_DEBUG) aggregator.traceAll=true;
	 // if (LOG_DEBUG) aggregator.traceTuples=true;
	
	 /**
	  * Use subelement name as the key
	  * @param {PV.Record} record
	  */
     aggregator.keyFunction=function(record)
     {
         return record.element.name + "|_|" + record.ObjectID;
     }

	 /**
	  * Initialize the state for the first record seen
	  * @param {PV.Record} record
	  */
     aggregator.initializeFunction=function(record) 
     {
         var result=new Object();
         this.initializeState(result);
         return result;
     }
     
	 /**
	  * Calculate state for record
	  * @param {String} key
	  * @param {Object} state
	  * @param {PV.Record} record
	  */
    aggregator.calculateFunction=function(key, state, record) 
     {
          if (isValid(record.aggSettings)) 
          {
		  		logMessage(7, logComponent, "aggregator", "found good record.");
               if ( isValid(state))
               {
			  		logMessage(7, logComponent, "aggregator", "...record has a state");			   		
			   		if(isValid(state[record.CounterID]))
					{
						var delta = 0;
						if(isValid(record.Value))
						{
							delta = record.Value * 1 - state[record.CounterID];
						} else {
							if( ! record.aggSettings.reportZeros) {
								return;
							}
						}
						//TODO if delta negative check for daily rollover
						if(delta < 0) {
							//Check if this should have been a date rollover
							logMessage(5, logComponent, "aggregator", "Negative delta detected. subelt: "+key+" oldval: " + state[record.CounterID] +  " Skipping: "+ record );
							if(isValid(record.Value))
							{
								state[record.CounterID] = record.Value;
							} else {
								if(record.aggSettings.reportZeros) {
									state[record.CounterID] = 0;
									addAggMetric(this.parentOperator, record.aggSettings.formulaPath, 0, record.timestamp,record.subelement.name);
								}
							}
						} else {
						//TODO else make metric for data and assign previous
							addAggMetric(this.parentOperator, record.aggSettings.formulaPath, delta, record.timestamp,record.subelement.name);
							if(delta > 0)
							{
								//If delta==0 then the previous value doesn't need to be overwritten it's the same
								state[record.CounterID] = record.Value;
							}
						}
					} else {
						// aggSettings is valid, the state exists, however the counter never had a previous value 
						if(isValid(record.Value))
						{
							//Sets the in-memory value to the record value for next use
							state[record.CounterID] = record.Value;
							logMessage(5, logComponent, "aggregator", "..." +key+" has no state for object "+record.CounterID+", setting =" + record.Value);
						} else {
							if(record.aggSettings.reportZeros) {
								// This is a special case where we want to record this even if there's not a valid counter value,
								//   placing a zero as a place holder for the next use.
								state[record.CounterID] = 0;
								logMessage(5, logComponent, "aggregator", "..." +key+" has no state for object "+record.CounterID+", setting = 0");
							}
						}
					}
               }
          }
    }

	 /**
	  *  The period is closed and we need to flush our metrics and reset our state for the next period
	  * @param {Object} state
	  */
     aggregator.closePeriodFunction=function(state) 
     {
          //TODO figure out if this is the End of Day, if so set the previous daily
		  // rollover to the counter
		  state.lastTimestamp= this.startOfNextPeriod - 1;
     }

	 /**
	  * Initialize the state
	  * @param {Object} state
	  */
     aggregator.initializeState=function(state)
     {
		//set the state to never for this resource.
		state.lastTimestamp = -1;
     }


//     result.keyFunction=logBeforeAfter("keyFunction",result.keyFunction);
//     result.initializeFunction=logBeforeAfter("initializeFunction",result.initializeFunction);
//     result.calculateFunction=logBeforeAfter("calculateFunction",result.calculateFunction);
//     result.closePeriodFunction=logBeforeAfter("closePeriodFunction",result.closePeriodFunction);
//     result.initializeState=logBeforeAfter("initializeState",result.initializeState);
//     result.initializeWithInventory=logBeforeAfter("initializeWithInventory",result.initializeWithInventory);

	return aggregator;
	
}

/**
 * Adds the aggregating metric to the operator by looking up the subelement and creating the metric 
 * using the rest of the input
 * @param {PV.Operator} operator
 * @param {String} formulaPath
 * @param {String} value
 * @param {Date} timestamp
 * @param {String} subelementName
 */
function addAggMetric(operator, formulaPath, value, timestamp, subelementName) {	
	if( !isValid(value) )
          return;
	var subelement = operator.subelementNamed(subelementName);
	if( subelement == null) {
			logMessage(5, logComponent, "addAggMetric", "FAIL: subelementNamed returned undefined");
			return;	  
			}
	addMetricWithOperator(operator,subelement, formulaPath, timestamp, value);
}
